#ifndef INCLUDED_ssl_h
#define INCLUDED_ssl_h

#include <openssl/ssl.h>
#include <openssl/err.h>

/* pre-declare this, we can't include client.h yet */
struct Client;

typedef enum {
	SSLC_STATE_NONE,
	SSLC_STATE_HANDSHAKE,
	SSLC_STATE_CONNECTED,
	SSLC_STATE_STARTTLS,
	SSLC_STATE_DEAD
} sslc_state;

#define SSLC_HANDSHAKE_NOERROR		0
#define SSLC_HANDSHAKE_INVALID_PARAM	-1
#define SSLC_HANDSHAKE_NEW_FAIL		-2
#define SSLC_HANDSHAKE_SETFD_FAIL	-3
#define SSLC_HANDSHAKE_SSLERROR		-4
#define SSLC_HANDSHAKE_ACCEPT_FAIL	-5

#define SSLC_STAT_NONE			0x00
#define SSLC_STAT_KEY_PUBLIC		0x01
#define SSLC_STAT_KEY_PRIVATE		0x02
#define SSLC_STAT_KEY_BOTH		(SSLC_STAT_KEY_PUBLIC | SSLC_STAT_KEY_PRIVATE)
#define SSLC_STAT_STARTTLS		0x04
#define SSLC_STAT_STARTTLS_OK		(SSLC_STAT_KEY_BOTH | SSLC_STAT_STARTTLS)

extern void SSLC_CTX_init(void);
extern void SSLC_CTX_deinit(void);
extern int SSLC_handshake(struct Client *);
extern int SSLC_write(struct Client *, const void *, int);
extern int SSLC_read(struct Client *, void *, int);
extern void SSLC_shutdown(struct Client *);
extern void SSLC_CTX_deinit(void);
extern const char *SSLC_cipherinfo(struct Client *, int);
extern void SSLC_CertificateFile(const char *);
extern void SSLC_PrivateKeyFile(const char *);
#endif
