/*
 * m_starttls.c: Client STARTTLS command support.
 *
 * Copyright (c) 2004  Ben Winslow.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    the above paragraph (the one permitting redistribution), this list of
 *    conditions and the following disclaimer in the documentation and/or
 *    other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id$
 */

#include "config.h"

#ifdef SSL_CLIENT_SUPPORT
#include "stdinc.h"
#include "ssl_client.h"
#include "msg.h"
#include "handlers.h"
#include "client.h"
#include "s_conf.h"
#include "isupport.h"
#include "hook.h"
#include "parse.h"
#include "send.h"
#include "modules.h"

static void m_starttls(struct Client *, struct Client *, int, char **);

struct Message starttls_msgtab = {
	"STARTTLS", 0, 0, 0, 0, MFLG_SLOW | MFLG_UNREG, 0, 
	{m_starttls, m_starttls, m_ignore, m_starttls}
};

#ifndef STATIC_MODULES
void _modinit(void)
{
	mod_add_cmd(&starttls_msgtab);
	
	ServerInfo.ssl_status |= SSLC_STAT_STARTTLS;
	hook_call_event("update_isupport", ISUP_PROPAGATE);
}

void _moddeinit(void)
{
	mod_del_cmd(&starttls_msgtab);
	
	ServerInfo.ssl_status &= ~SSLC_STAT_STARTTLS;
	hook_call_event("update_isupport", ISUP_PROPAGATE);
}

const char *_version = "$Revision: 1.0 $";
#endif

/*
 * m_starttls - STARTTLS command handler
 */
static void m_starttls(struct Client *client_p, struct Client *source_p,
	int parc, char *parv[])
{
	/* this shouldn't ever happen */
	if (!MyConnect(source_p))
		return;

	if (source_p->localClient->ssl_state != SSLC_STATE_NONE) {
		sendto_one(source_p, "NOTICE %s :TLS/SSL already started: %s",
			IsRegistered(source_p) ? source_p->name : "TLS",
			SSLC_cipherinfo(source_p, 0));
		return;
	}

	sendto_one(source_p, "NOTICE %s :Ready to start TLS",
		IsRegistered(source_p) ? source_p->name : "TLS");
	
	source_p->localClient->ssl_state = SSLC_STATE_STARTTLS;
}
#endif
